//classic includes (some may not be used)
#include "gpdef.h"
#include "gpstdlib.h"
#include "gpgraphic.h"
#include "gpmain.h"
#include "gpfont.h"
#include "gpstdio.h"

#include <math.h>
#include <limits.h>

//Size of the screen
#define LCD_WIDTH 320
#define LCD_HEIGHT 240
//Size of a character in pixels
#define WIDTH_CHAR 8
#define HEIGHT_CHAR 11

//temp string...
static char strtmp [255];

//constant used to display things at the correct place

const int xTitle = 55;
const int yTitle = 10;
const int xPlayerName = 10;
const int xComputerName = 250;
const int yName = 110;
const int xLine = 160;
const int yZero = 200;
const int yHundred = 80;
const int xPressKeyText = 25;
const int yPressKeyText = 220;

const	int xPlayer = 110;
const	int xComputer = 185;
const int xGoal = 160-1.5*WIDTH_CHAR;
const int yGoal = 30;

//return a value between 0 and 100
inline int rand100 ()
{
	return GpRandN(101);
}

//return the Y coordinate from a value between 0 and 100
//I agree this is not easily readable. Just trust me when I says that the formulae is correct.
inline int YfromValue(int value)
{
	return ((100-(float)value)/100.)*(yZero-yHundred)+yHundred - HEIGHT_CHAR/2.;

}

//Draw title, vertical lines and the boxes (goal, player, computer)
// if goal is -1, there are ??? displayed
// if computer is -1, its box is not displayed
void drawScreen (GPDRAWSURFACE * screen, int goal, int player, int computer)
{
	int yPlayer;
	int yComputer;

	//clear screen (0xFF = white)
	gm_memset(screen->ptbuffer, 0xFF, LCD_WIDTH*LCD_HEIGHT);
	GpTextOut(NULL, screen, xTitle, yTitle, "Guess The Hidden Number...", 0x00);

	if (goal != -1)
	sprintf(strtmp, "%3i", goal); // display goal value if different than -1
	else
	gm_strcpy(strtmp, "???"); // else display "???"
	GpTextOut(NULL, screen, xGoal, yGoal, strtmp, 0x00);
	GpRectDraw(screen, xGoal - 2, yGoal, xGoal + 2 + 3*WIDTH_CHAR, yGoal + 2 + HEIGHT_CHAR, 0x00);


	//display texts: "Player" and "GP32"
	GpTextOut(NULL, screen, xPlayerName, yName, "Player", 0x00);
	GpTextOut(NULL, screen, xComputerName, yName, "GP32", 0x00);

	//display player's box
	sprintf(strtmp, "%3i", player);
	GpTextOut(NULL, screen, xPlayer, YfromValue(player), strtmp, 0x00);
	GpRectDraw(screen, xPlayer - 2, YfromValue(player), xPlayer + 2 + 3*WIDTH_CHAR, YfromValue(player) + 2 + HEIGHT_CHAR, 0x00);
	GpLineDraw(screen, xPlayer + 2 + 3*WIDTH_CHAR, YfromValue(player)+6, xLine, YfromValue(player)+6, 0x00);

	if (computer != -1)
	{
		//display computer's box
		sprintf(strtmp, "%3i", computer);
		GpTextOut(NULL, screen, xComputer, YfromValue(computer), strtmp, 0x00);
		GpRectDraw(screen, xComputer - 2, YfromValue(computer), xComputer + 2 + 3*WIDTH_CHAR, YfromValue(computer) + 2 + HEIGHT_CHAR, 0x00);
		GpLineDraw(screen, xComputer - 2, YfromValue(computer)+6, xLine, YfromValue(computer)+6, 0x00);
	}

	//display vertical line 0-100
	GpLineDraw(screen, xLine - WIDTH_CHAR, yHundred, xLine+WIDTH_CHAR, yHundred, 0x00);
	GpLineDraw(screen, xLine - WIDTH_CHAR, yZero, 160+WIDTH_CHAR, yZero, 0x00);
	GpLineDraw(screen, xLine , yHundred, xLine, yZero, 0x00);

	GpTextOut(NULL, screen, xLine-1.5*WIDTH_CHAR,  yHundred - HEIGHT_CHAR-3, "100", 0x00);
	GpTextOut(NULL, screen, xLine-1.5*WIDTH_CHAR, yZero, " 0 ", 0x00);
}

void GpMain(void *arg)
{
	int key;
	unsigned int ticks; //used for random number initialization
	int i, j; // used in some loops
	int goal, player=50, computer; // value of the three boxes
	int playerScore, computerScore; // the score is the distance between the goal and the value. 0 is the best score.
	int smallAllowed; //used for shoulder buttons. One press, one increase.

	//init screen
	GpGraphicModeSet(8, NULL); //8bits , standard palette

	//create and initialize the 2 framebuffers
	GPDRAWSURFACE *gpOffDraw;
	int currentScreen=1;
	gpOffDraw = (GPDRAWSURFACE*) gm_malloc(2*sizeof(GPDRAWSURFACE));
	GpLcdSurfaceGet(&(gpOffDraw[0]),0);
	GpLcdSurfaceGet(&(gpOffDraw[1]),1);
	gm_memset((gpOffDraw[0]).ptbuffer, 0x00, LCD_WIDTH*LCD_HEIGHT);
	gm_memset((gpOffDraw[1]).ptbuffer, 0x00, LCD_WIDTH*LCD_HEIGHT);

	//assign the first buffer to the screen
	GpTextOut(NULL, &(gpOffDraw[0]), 10, 100, "Press START to initialize random seed.", 0xFF);
	GpSurfaceSet(&(gpOffDraw[0]));

	//start key handling
	GpKeyInit();

	//init random value
	do{key = GpKeyGet();}
	while ((key&GPC_VK_START) != GPC_VK_START);

	ticks = GpTickCountGet();
	GpSrand ((int)ticks);

	//Main loop.
	while(1)
	{
		smallAllowed = 1; //initialize value of smallAllowed to 1 (default value when no shoulder is pressed.
		//set hidden value to find
		goal = rand100();

		//ask player value
		do{
			//draw most of the screen
			drawScreen(&(gpOffDraw[currentScreen]), -1, player, -1);
			//display "press 'A' when ok"
			GpTextOut(NULL, &(gpOffDraw[currentScreen]), xPressKeyText, yPressKeyText, "Press A to validate your choice...", 0x00);


			//display frame
			GpSurfaceFlip (&(gpOffDraw[currentScreen]));
			currentScreen = 1 - currentScreen;


			key = GpKeyGet();
			if ((key&GPC_VK_LEFT) != 0)
			player-=10;
			if ((key&GPC_VK_RIGHT) != 0)
			player+=10;
			if ((key&GPC_VK_DOWN) != 0)
			player-=1;
			if ((key&GPC_VK_UP) != 0)
			player+=1;

			//smallAllowed is a variable used to force the user to release the shoulder button between 2 "action"
			if (smallAllowed == 1)
			{
				if ((key&GPC_VK_FL) != 0)
				{
					player-=1;
					smallAllowed = 0;
				}
				else
				if ((key&GPC_VK_FR) != 0)
				{
					player+=1;
					smallAllowed = 0;
				}
			}
			if ((key&GPC_VK_FL) == 0 && (key&GPC_VK_FR) == 0)
			smallAllowed = 1;
			if (player < 0) player=0;
			if (player > 100) player=100;
		}
		while ((key&GPC_VK_FA) == 0);

		//Wait the releasing of A
		while 	((key&GPC_VK_FA) != 0)
		{key = GpKeyGet();}


		//just to be sure that everything is well drawn
		drawScreen(&(gpOffDraw[currentScreen]), -1, player, -1);
		GpSurfaceFlip (&(gpOffDraw[currentScreen]));
		currentScreen = 1 - currentScreen;

		//set computer value
		for (i=0; i<22; i++)
		{
			computer = rand100();
			drawScreen(&(gpOffDraw[currentScreen]), -1, player, computer);
			GpSurfaceFlip (&(gpOffDraw[currentScreen]));
			currentScreen = 1 - currentScreen;

			for (j=0; j< 3000*i; j++) rand100(); //just wait...
		}

		//Final draw
		drawScreen(&(gpOffDraw[currentScreen]), goal, player, computer);
		//draws goal line
		GpLineDraw(&(gpOffDraw[currentScreen]), xLine - WIDTH_CHAR, YfromValue(goal)+6, xLine+WIDTH_CHAR, YfromValue(goal)+6, 0x00);

		//draws player column
		// "if" used because the gp32 needs absolutly the upper left coordinate of the box first
		if (player < goal )
		GpRectFill (NULL, &(gpOffDraw[currentScreen]), xLine - WIDTH_CHAR + 2, YfromValue(goal)+6, 3, YfromValue(player) - YfromValue(goal), 0x00);
		else
		GpRectFill (NULL, &(gpOffDraw[currentScreen]), xLine - WIDTH_CHAR + 2, YfromValue(player)+6, 3, YfromValue(goal) - YfromValue(player), 0x00);
		//draws computer column
		if (computer < goal)
		GpRectFill (NULL, &(gpOffDraw[currentScreen]), xLine + 4, YfromValue(goal)+6, 3, YfromValue(computer) - YfromValue(goal), 0x00);
		else
		GpRectFill (NULL, &(gpOffDraw[currentScreen]), xLine + 4, YfromValue(computer)+6, 3, YfromValue(goal) - YfromValue(computer), 0x00);

		//decide win/lose
		playerScore = fabs(goal - player);
		computerScore = fabs(goal - computer);
		if (playerScore == computerScore)
		{
			GpTextOut(NULL, &(gpOffDraw[currentScreen]), xPlayerName, yName + HEIGHT_CHAR, "draws.", 0x00);
			GpTextOut(NULL, &(gpOffDraw[currentScreen]), xComputerName, yName + HEIGHT_CHAR, "draws.", 0x00);
		}
		if (playerScore > computerScore)
		{
			GpTextOut(NULL, &(gpOffDraw[currentScreen]), xPlayerName, yName + HEIGHT_CHAR, "loses.", 0x00);
			GpTextOut(NULL, &(gpOffDraw[currentScreen]), xComputerName, yName + HEIGHT_CHAR, "wins.", 0x00);
		}
		if (playerScore < computerScore)
		{
			GpTextOut(NULL, &(gpOffDraw[currentScreen]), xPlayerName, yName + HEIGHT_CHAR, "wins.", 0x00);
			GpTextOut(NULL, &(gpOffDraw[currentScreen]), xComputerName, yName + HEIGHT_CHAR, "loses.", 0x00);
		}
		if (playerScore == 0)
		GpTextOut(NULL, &(gpOffDraw[currentScreen]), xPlayerName, yName + 3*HEIGHT_CHAR, "(PERFECT)", 0x00);
		if (computerScore == 0)
		GpTextOut(NULL, &(gpOffDraw[currentScreen]), xComputerName, yName + 3*HEIGHT_CHAR, "(PERFECT)", 0x00);

		GpTextOut(NULL, &(gpOffDraw[currentScreen]), xPressKeyText, yPressKeyText, "      Press A to continue...      ", 0x00);
		GpSurfaceFlip (&(gpOffDraw[currentScreen]));
		currentScreen = 1 - currentScreen;

		//wait key press
		do
		{
			key = GpKeyGet();
		}
		while ((key&GPC_VK_FA) == 0);

		//Wait the releasing of A
		while 	((key&GPC_VK_FA) != 0)
		{key = GpKeyGet();}

		//and loops
	};
	//this part of the code is currently never read but it may change in the future.

	//RESET
	asm volatile("swi #4\n");
}
